#!/usr/bin/env perl

package yahoo_finance;

require Exporter;
@ISA =qw(Exporter);
@EXPORT = qw( scan_market_links );
$VERSION = '0.60';

use strict;
use DBI;
use LWP::UserAgent;

=head1 USAGE

yahoo_finance.pm

is a module useful to download the quote data of the 
stocks of your favourite markets
from the YAHOO web site and then store them in a MySql database.
Run this script every day and build the history of the prices of 
the stocks... You'll be able to create your custom charts...


=head1 REQUIREMENT

Perl 5 with modules DBI and LWP, a MySql database, an Internet 
connection (if you have a proxy, set it via the enviroment http_proxy).

=head1 INSTALLATION & CONFIGURATION

Create a new MySql database with the name "finance"

	mysqladmin -u root create finance

Create the tables with
	
	mysql -u root finance <finance.sql


To insert a new market, add an entry in the "markets" table (choose
the same short name Yahoo uses, for instance AEX,BFX,GDAXI,MIB30)
and then the link of the .cvs file(s) in the table "market_link".

In  other words...

Open a browser and go to the page of YAHOO 
with the quote data of the market you want to add (you can start from 
http://finance.yahoo.com/?u or for the Europe people  
http://it.biz.yahoo.com/eu/). Find the link with the description 
"Download Spreadsheet" that let you
to store the quotes in a file .csv and add that link in the table 
"market_link"



=head1 TESTED

Tested on Debian Linux 3.0 and perl 5.6.1, but it should work everywhere
perl and mysql run (HP-UX, Solaris, Windows, FreeBSD, ...)


=head1 TODO

Some scripts with a web interface to create and view charts...


=head1 HISTORY

2002-08-13 Matteo: automatic population of the (so far unused) tables market_stocks and stocks
2002-08-12 Matteo: first rows of this script


=head1 AUTHOR

E-MAIL: matteo.redaelli@libero.it

WEB:    http://digilander.iol.it/reda


=head1 LICENSE

This package is free software; you can redistribute it and/or
modify it under the same terms as Perl itself, i.e., under the
terms of the "Artistic License" or the "GNU General Public License".

The C library at the core of this Perl module can additionally
be redistributed and/or modified under the terms of the
"GNU Library General Public License".


=head1 DISCLAIMER

This package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the "GNU General Public License" for more detail

=cut

sub db_connect
{
        return DBI->connect("DBI:mysql:database=finance;host=localhost",
                         "root", "",
                         {PrintError => 0});
#                         {PrintError => 0 /*, 'RaiseError' => 1  */});
}

sub db_disconnect
{
        my $dbh = @_[0];

        $dbh->disconnect();
}

sub db_request_ddl
{
	my $dbh = db_connect() or return $DBI::errstr;

        eval { $dbh->do("@_[0]") } or return $DBI::errstr;
#       print "SQL request @_[0] failed.\n" if $@;
        db_disconnect($dbh ) or return $DBI::errstr;

	return 0;
}

sub get_webpage
{

	my $ua = LWP::UserAgent->new;
	$ua->env_proxy; 


	my $url = @_[0];
	my $req = HTTP::Request->new(GET => $url);
# 	print $ua->request($req)->as_string;
	return $ua->request($req)->as_string;
}

sub scan_webpage
{

	my $page = get_webpage(@_[0]);
	my $market_id = @_[1];

	my @rows =  split( "\n",  $page ) ;

	# { }

	my $found = 0;

	foreach my $r ( @rows )  { 
		
		# skip until an empty line is found 
		
		if ( $r =~ /^ *$/ )  { 
			$found = 1;
			next;
		}

		# skip the last line
		
		# next if $found == 0 or ! ($r =~ /,/) or ($r =~ /^"\^/);
		next if $found == 0 or ! ($r =~ /,/);

		#  print "$r \n" if ($found);
		my @fields =  split(",", $r);
		$fields[2] =~ /"(\d+)\/(\d+)\/(\d\d\d\d)"/ and
			my $mydate = "$3-$1-$2";

		my $sql_update = 
			"update quote_data set quote = $fields[1], the_time = $fields[3] where stock_id=$fields[0] and the_day='$mydate'";
		my $sql_insert =  
			"insert into quote_data ( stock_id, the_day, the_time, quote) values( $fields[0],'$mydate', $fields[3], $fields[1] )";

		# add the stock to the table stocks if it doesn't exist
		
		my $sql =  
			"insert into stocks ( stock_id) values(  $fields[0] ) ";
		db_request_ddl("$sql");

		# add the stock to the table market_stocks if it doesn't exist
		
		my $sql =  
			"insert into market_stocks ( market_id, stock_id) values( $market_id, $fields[0] ) ";
		db_request_ddl("$sql");

#	print "$fields[0]\n";
#
	db_request_ddl("$sql_update");
	db_request_ddl("$sql_insert");

	}
}

sub scan_market_links
{
	my $market_id = @_[0];
	my $dbh = db_connect();


	# iterator on all links of the market
	
	my $sql = "SELECT link FROM market_links";
	$sql = "$sql WHERE market_id = $market_id" if  $market_id;

	my $sth = $dbh->prepare($sql);
	
	$sth->execute();

	while ( my @row = $sth->fetchrow_array ) {
		print "@row[0]\n";
		scan_webpage(@row[0], $market_id );
         }
        db_disconnect($dbh );

}

